/*
 * @author Emanuele Rodola 21/09/2012
 * fixed 15/07/2014
 */

#include "mex.h"
#include <vector>

//DEBUG
#include <fstream>
#include <iostream>

void prune_matches(const double* x, int n, const double* candidate_matches, std::vector<int>& matches)
{
	std::vector<bool> is_active(n, true);
	
	int best_match = -1;
	double best_match_val = 0.;

	for (int i=0; i<n; ++i)
	{
		const double xi = x[i];
		if (xi == 0.)
			is_active[i] = false;
		if (xi > best_match_val)
		{
			best_match_val = xi;
			best_match = i;
		}
	}
	
	while (best_match != -1)
	{
		is_active[best_match] = false;
		matches.push_back(best_match);
		
		for (int i=0; i<n; ++i)
		{
			if (is_active[i])
			{
				if ((candidate_matches[2*i] == candidate_matches[2*best_match]) || 
				    (candidate_matches[2*i+1] == candidate_matches[2*best_match+1]))
				is_active[i] = false;
			}
		}
		
		best_match = -1;
		best_match_val = 0.;
		
		for (int i=0; i<n; ++i)
		{
			if (is_active[i])
			{
				const double xi = x[i];
				if (xi > best_match_val)
				{
					best_match_val = xi;
					best_match = i;
				}
			}
		}
	}
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray*prhs[])
{
	if (nrhs != 2 || nlhs != 1)
      mexErrMsgTxt("Usage: nmatches = prune_matches(x, candidate_matches).");

	const double* const x = mxGetPr(prhs[0]);
	const int n = int( mxGetM(prhs[0]) );
	const double* const candidate_matches = mxGetPr(prhs[1]);
	
	if (int( mxGetM(prhs[1]) ) != 2)
		mexErrMsgTxt("The candidate matches must be passed as a n-by-2 matrix (maybe you forgot to transpose?)");
	
	std::vector<int> matches;
	prune_matches(x, n, candidate_matches, matches);
	
	plhs[0] = mxCreateDoubleMatrix(matches.size(), 1, mxREAL);
    double* p = mxGetPr(plhs[0]);
	
	for (size_t i=0; i<matches.size(); ++i)
		p[i] = static_cast<double>(matches[i])+1;
}
